@@ -13,7 +13,7 @@
 #include <functional>
 #include <type_traits>
 
-#include "xtl/xsequence.hpp"
+#include <xtl/xsequence.hpp>
 
 #include "xconcepts.hpp"
 #include "xfunction.hpp"
@@ -246,7 +246,7 @@ namespace xt
         template <template <class...> class F, class... E>
         inline auto make_xfunction(E&&... e) noexcept
         {
-            using function_type = xfunction_type<F, E... >;
+            using function_type = xfunction_type<F, E...>;
             using functor_type = typename function_type::functor_type;
             using type = typename function_type::type;
             return type(functor_type(), std::forward<E>(e)...);
@@ -388,11 +388,11 @@ namespace xt
      */
     template <class E1, class E2>
     inline auto operator%(E1&& e1, E2&& e2) noexcept
-    -> detail::xfunction_type_t<detail::modulus, E1, E2>
+        -> detail::xfunction_type_t<detail::modulus, E1, E2>
     {
         return detail::make_xfunction<detail::modulus>(std::forward<E1>(e1), std::forward<E2>(e2));
     }
-    
+
     /**
      * @defgroup logical_operators Logical operators
      */
@@ -763,16 +763,8 @@ namespace xt
     inline bool any(E&& e)
     {
         using xtype = std::decay_t<E>;
-        if (xtype::static_layout == layout_type::row_major || xtype::static_layout == layout_type::column_major)
-        {
-            return std::any_of(e.cbegin(), e.cend(),
-                               [](const typename std::decay_t<E>::value_type& el) { return el; });
-        }
-        else
-        {
-            return std::any_of(e.begin(), e.end(),
-                               [](const typename std::decay_t<E>::value_type& el) { return el; });
-        }
+        using value_type = typename xtype::value_type;
+        return std::any_of(e.cbegin(), e.cend(), [](const value_type& el) { return el; });
     }
 
     /**
@@ -788,16 +780,8 @@ namespace xt
     inline bool all(E&& e)
     {
         using xtype = std::decay_t<E>;
-        if (xtype::static_layout == layout_type::row_major || xtype::static_layout == layout_type::column_major)
-        {
-            return std::all_of(e.cbegin(), e.cend(),
-                               [](const typename std::decay_t<E>::value_type& el) { return el; });
-        }
-        else
-        {
-            return std::all_of(e.begin(), e.end(),
-                               [](const typename std::decay_t<E>::value_type& el) { return el; });
-        }
+        using value_type = typename xtype::value_type;
+        return std::all_of(e.cbegin(), e.cend(), [](const value_type& el) { return el; });
     }
 
     /**
