@@ -15,7 +15,8 @@
 #include <type_traits>
 #include <utility>
 
-#include "xtl/xsequence.hpp"
+#include <xtl/xsequence.hpp>
+#include <xtl/xvariant.hpp>
 
 #include "xarray.hpp"
 #include "xexpression.hpp"
@@ -165,7 +166,7 @@ namespace xt
         const_reference element(It first, It last) const;
 
         template <class O>
-        bool broadcast_shape(O& shape) const;
+        bool broadcast_shape(O& shape, bool reuse_cache = false) const;
 
         template <class O>
         bool is_trivial_broadcast(const O& strides) const noexcept;
@@ -346,13 +347,13 @@ namespace xt
     template <class CT, class S, class CD>
     inline auto xstrided_view<CT, S, CD>::data() noexcept -> underlying_container_type&
     {
-        return m_e.data();
+        return m_data;
     }
 
     template <class CT, class S, class CD>
     inline auto xstrided_view<CT, S, CD>::data() const noexcept -> const underlying_container_type&
     {
-        return m_e.data();
+        return m_data;
     }
 
     template <class CT, class S, class CD>
@@ -525,7 +526,7 @@ namespace xt
      */
     template <class CT, class S, class CD>
     template <class O>
-    inline bool xstrided_view<CT, S, CD>::broadcast_shape(O& shape) const
+    inline bool xstrided_view<CT, S, CD>::broadcast_shape(O& shape, bool) const
     {
         return xt::broadcast_shape(m_shape, shape);
     }
@@ -652,146 +653,9 @@ namespace xt
     inline auto strided_view(E&& e, I&& shape, I&& strides, std::size_t offset = 0, layout_type layout = layout_type::dynamic) noexcept
     {
         using view_type = xstrided_view<xclosure_t<E>, I, decltype(e.data())>;
-        return view_type(std::forward<E>(e), std::forward<I>(shape), std::forward<I>(strides), offset);
+        return view_type(std::forward<E>(e), std::forward<I>(shape), std::forward<I>(strides), offset, layout);
     }
 
-    /****************************
-     * transpose implementation *
-     ****************************/
-
-    namespace detail
-    {
-        template <class E, class S>
-        inline auto transpose_impl(E&& e, S&& permutation, check_policy::none)
-        {
-            if (sequence_size(permutation) != e.dimension())
-            {
-                throw transpose_error("Permutation does not have the same size as shape");
-            }
-
-            // permute stride and shape
-            using strides_type = typename std::decay_t<E>::strides_type;
-            strides_type temp_strides;
-            resize_container(temp_strides, e.strides().size());
-
-            using shape_type = typename std::decay_t<E>::shape_type;
-            shape_type temp_shape;
-            resize_container(temp_shape, e.shape().size());
-
-            using size_type = typename std::decay_t<E>::size_type;
-            for (std::size_t i = 0; i < e.shape().size(); ++i)
-            {
-                if (std::size_t(permutation[i]) >= e.dimension())
-                {
-                    throw transpose_error("Permutation contains wrong axis");
-                }
-                size_type perm = static_cast<size_type>(permutation[i]);
-                temp_shape[i] = e.shape()[perm];
-                temp_strides[i] = e.strides()[perm];
-            }
-
-            layout_type new_layout = layout_type::dynamic;
-            if (std::is_sorted(std::begin(permutation), std::end(permutation)))
-            {
-                // keep old layout
-                new_layout = e.layout();
-            }
-            else if (std::is_sorted(std::begin(permutation), std::end(permutation), std::greater<>()))
-            {
-                // this swaps the layout
-                if (e.layout() == layout_type::row_major)
-                {
-                    new_layout = layout_type::column_major;
-                }
-                else if (e.layout() == layout_type::column_major)
-                {
-                    new_layout = layout_type::row_major;
-                }
-            }
-
-            using view_type = xstrided_view<xclosure_t<E>, shape_type, decltype(e.data())>;
-            return view_type(std::forward<E>(e), std::move(temp_shape), std::move(temp_strides), 0, new_layout);
-        }
-
-        template <class E, class S>
-        inline auto transpose_impl(E&& e, S&& permutation, check_policy::full)
-        {
-            // check if axis appears twice in permutation
-            for (std::size_t i = 0; i < sequence_size(permutation); ++i)
-            {
-                for (std::size_t j = i + 1; j < sequence_size(permutation); ++j)
-                {
-                    if (permutation[i] == permutation[j])
-                    {
-                        throw transpose_error("Permutation contains axis more than once");
-                    }
-                }
-            }
-            return transpose_impl(std::forward<E>(e), std::forward<S>(permutation), check_policy::none());
-        }
-    }
-
-    /**
-     * Returns a transpose view by reversing the dimensions of xexpression e
-     * @param e the input expression
-     */
-    template <class E>
-    inline auto transpose(E&& e) noexcept
-    {
-        using shape_type = typename std::decay_t<E>::shape_type;
-
-        shape_type shape;
-        resize_container(shape, e.shape().size());
-        std::copy(e.shape().rbegin(), e.shape().rend(), shape.begin());
-
-        shape_type strides;
-        resize_container(strides, e.strides().size());
-        std::copy(e.strides().rbegin(), e.strides().rend(), strides.begin());
-
-        layout_type new_layout = layout_type::dynamic;
-        if (e.layout() == layout_type::row_major)
-        {
-            new_layout = layout_type::column_major;
-        }
-        else if (e.layout() == layout_type::column_major)
-        {
-            new_layout = layout_type::row_major;
-        }
-
-        using view_type = xstrided_view<xclosure_t<E>, shape_type, decltype(e.data())>;
-        return view_type(std::forward<E>(e), std::move(shape), std::move(strides), 0, new_layout);
-    }
-
-    /**
-     * Returns a transpose view by permuting the xexpression e with @p permutation.
-     * @param e the input expression
-     * @param permutation the sequence containing permutation
-     * @param check_policy the check level (check_policy::full() or check_policy::none())
-     * @tparam Tag selects the level of error checking on permutation vector defaults to check_policy::none.
-     */
-    template <class E, class S, class Tag = check_policy::none>
-    inline auto transpose(E&& e, S&& permutation, Tag check_policy = Tag())
-    {
-        return detail::transpose_impl(std::forward<E>(e), std::forward<S>(permutation), check_policy);
-    }
-
-    /// @cond DOXYGEN_INCLUDE_SFINAE
-#ifdef X_OLD_CLANG
-    template <class E, class I, class Tag = check_policy::none>
-    inline auto transpose(E&& e, std::initializer_list<I> permutation, Tag check_policy = Tag())
-    {
-        dynamic_shape<I> perm(permutation);
-        return detail::transpose_impl(std::forward<E>(e), std::move(perm), check_policy);
-    }
-#else
-    template <class E, class I, std::size_t N, class Tag = check_policy::none>
-    inline auto transpose(E&& e, const I (&permutation)[N], Tag check_policy = Tag())
-    {
-        return detail::transpose_impl(std::forward<E>(e), permutation, check_policy);
-    }
-#endif
-    /// @endcond
-
     namespace detail
     {
         template <class CT>
@@ -805,6 +669,7 @@ namespace xt
             using size_type = typename xexpression_type::size_type;
             using value_type = typename xexpression_type::value_type;
             using reference = typename xexpression_type::reference;
+            using const_reference = typename xexpression_type::const_reference;
 
             using iterator = typename xexpression_type::iterator;
             using const_iterator = typename xexpression_type::const_iterator;
@@ -815,18 +680,26 @@ namespace xt
                 resize_container(m_index, m_e.dimension());
                 resize_container(m_strides, m_e.dimension());
                 m_size = compute_size(m_e.shape());
-                compute_strides(m_e.shape(), DEFAULT_LAYOUT, m_strides);
+                m_layout = m_e.layout();
+                compute_strides(m_e.shape(), m_layout, m_strides);
             }
 
-            const reference operator[](std::size_t idx) const
+            expression_adaptor(CT&& e, shape_type&& strides, layout_type layout)
+                : m_e(e), m_strides(std::move(strides)), m_layout(layout)
             {
-                std::size_t quot;
-                for (size_type i = 0; i < m_strides.size(); ++i)
-                {
-                    quot = idx / m_strides[i];
-                    idx = idx % m_strides[i];
-                    m_index[i] = quot;
-                }
+                resize_container(m_index, m_e.dimension());
+                m_size = e.size();
+            }
+
+            reference operator[](std::size_t idx)
+            {
+                m_index = detail::unravel_noexcept(idx, m_strides, m_layout);
+                return m_e.element(m_index.cbegin(), m_index.cend());
+            }
+
+            const_reference operator[](std::size_t idx) const
+            {
+                m_index = detail::unravel_noexcept(idx, m_strides, m_layout);
                 return m_e.element(m_index.cbegin(), m_index.cend());
             }
 
@@ -841,150 +714,58 @@ namespace xt
             shape_type m_strides;
             mutable index_type m_index;
             size_type m_size;
+            layout_type m_layout;
+        };
+
+        template <class E>
+        struct slice_getter_impl {
+            E& m_expr;
+            mutable std::size_t idx;
+
+            slice_getter_impl(E& expr)
+                : m_expr(expr), idx(0)
+            {
+            }
+
+            template <class T>
+            std::array<int, 3> operator()(const T& t) const
+            {
+                auto sl = get_slice_implementation(m_expr, t, idx);
+                return std::array<int, 3>({int(sl(0)), int(sl.size()), int(sl.step_size())});
+            }
+
+            std::array<int, 3> operator()(const int& /*t*/) const
+            {
+                return std::array<int, 3>({0, 0, 0});
+            }
         };
     }
 
+
+    template <class T>
+    using slice_variant = xtl::variant<
+        T,
+
+        xt::xrange_adaptor<xt::placeholders::xtuph, T, T>,
+        xt::xrange_adaptor<T, xt::placeholders::xtuph, T>,
+        xt::xrange_adaptor<T, T, xt::placeholders::xtuph>,
+
+        xt::xrange_adaptor<T, xt::placeholders::xtuph, xt::placeholders::xtuph>,
+        xt::xrange_adaptor<xt::placeholders::xtuph, T, xt::placeholders::xtuph>,
+        xt::xrange_adaptor<xt::placeholders::xtuph, xt::placeholders::xtuph, T>,
+
+        xt::xrange_adaptor<T, T, T>,
+        xt::xrange_adaptor<xt::placeholders::xtuph, xt::placeholders::xtuph, xt::placeholders::xtuph>,
+
+        xt::xall_tag,
+        xt::xnewaxis_tag
+    >;
+
     /**
-     * The slice_vector is used as part of the dynamic_view interface.
-     * It offers an interface to dynamically push_back or append slices
-     * to dynamically create a view into a xexpression.
-     *
-     * This data structure is necessary as the slices of xview are
-     * compile-time static. If you do not know how many dimensions
-     * your expression has at compile-time, using dynamic_view can be
-     * useful.
-     *
-     * Internally, the dynamic_view is implemented using the strided_view
-     * adapter, which is very fast for data structures in contigous
-     * memory (such as xarrays and xtensors).
-     *
-     * Usage example:
-     *
-     * \code{.cpp}
-     * xt::xarray<double> a = {{1, 2, 3}, {4, 5, 6}};
-     * xt::slice_vector sv(a, xt::range(0, 1));
-     * sv.push_back(xt::all());
-     * \endcode
+     * @typedef slice_vector
+     * @brief vector of slices used to build a `xstrided_view`
      */
-    class slice_vector : private std::vector<std::array<long int, 3>>
-    {
-    public:
-
-        using index_type = long int;
-        using base_type = std::vector<std::array<index_type, 3>>;
-
-        // propagating interface
-        using base_type::begin;
-        using base_type::cbegin;
-        using base_type::end;
-        using base_type::cend;
-        using base_type::size;
-        using base_type::operator[];
-        using base_type::push_back;
-        using base_type::pop_back;
-
-        inline slice_vector() = default;
-
-        /**
-         * Constructor for slice vector
-         *
-         * @param e xexpression
-         * @param args xslices, integer, all or newaxis
-         */
-        template <class E, class... Args>
-        inline slice_vector(const xexpression<E>& e, Args... args)
-        {
-            const auto& de = e.derived_cast();
-            m_shape.resize(de.shape().size());
-            std::copy(de.shape().cbegin(), de.shape().cend(), m_shape.begin());
-            append(args...);
-        }
-
-        /**
-         * @brief Alternative constructor taking a shape
-         * @param shape the shape
-         */
-        template <class... Args>
-        inline slice_vector(const dynamic_shape<std::size_t>& shape, Args... args)
-        {
-            m_shape = shape;
-            append(args...);
-        }
-
-        /**
-         * Convenience function to append one or multiple
-         * slices to the slice_vector
-         *
-         * \code{.cpp}
-         * xt::slice_vector sv(a, xt::range(0, 1));
-         * sv.append(xt::all(), xt::newaxis(), 0, 12));
-         * \endcode
-         */
-        template <class T, class... Args>
-        inline void append(const T& s, Args... args)
-        {
-            push_back(s);
-            append(args...);
-        }
-
-        ///@cond DOXYGEN_INCLUDE_SFINAE
-        inline void append()
-        {
-        }
-        /// @endcond
-
-        /**
-         * Push back a single slice, integer, all or newaxis.
-         *
-         * @param s slice
-         */
-        template <class T>
-        inline void push_back(const xslice<T>& s)
-        {
-            auto ds = s.derived_cast();
-            base_type::push_back({index_type(ds(0)), index_type(ds.size()), index_type(ds.step_size())});
-        }
-
-        ///@cond DOXYGEN_INCLUDE_SFINAE
-        template <class A, class B, class C>
-        inline void push_back(const xrange_adaptor<A, B, C>& s)
-        {
-            auto idx = size() - newaxis_count;
-            if (idx >= m_shape.size())
-            {
-                throw std::runtime_error("Too many slices in slice vector for shape");
-            }
-            auto ds = s.get(m_shape[idx]);
-            base_type::push_back({index_type(ds(0)), index_type(ds.size()), index_type(ds.step_size())});
-        }
-
-        inline void push_back(xall_tag /*s*/)
-        {
-            auto idx = size() - newaxis_count;
-            if (idx >= m_shape.size())
-            {
-                throw std::runtime_error("Too many slices in slice vector for shape");
-            }
-            base_type::push_back({0, index_type(m_shape[idx]), 1});
-        }
-
-        inline void push_back(xnewaxis_tag /*s*/)
-        {
-            ++newaxis_count;
-            base_type::push_back({-1, 0, 0});
-        }
-
-        inline void push_back(index_type i)
-        {
-            base_type::push_back({i, 0, 0});
-        }
-        /// @endcond
-
-    private:
-
-        dynamic_shape<std::size_t> m_shape;
-        std::size_t newaxis_count = 0;
-    };
+    using slice_vector = std::vector<slice_variant<int>>;
 
     namespace detail
     {
@@ -1039,29 +820,35 @@ namespace xt
      *
      * \code{.cpp}
      * xt::xarray<double> a = {{1, 2, 3}, {4, 5, 6}};
-     * xt::slice_vector sv(a, xt::range(0, 1));
+     * xt::slice_vector sv({xt::range(0, 1)});
      * sv.push_back(xt::range(0, 3, 2));
      * auto v = xt::dynamic_view(a, sv);
      * // ==> {{1, 3}}
      * \endcode
+     * 
+     * You can also achieve the same with the following short-hand syntax:
+     * 
+     * \code{.cpp}
+     * xt::xarray<double> a = {{1, 2, 3}, {4, 5, 6}};
+     * auto v = xt::dynamic_view(a, {xt::range(0, 1), xt::range(0, 3, 2)});
+     * // ==> {{1, 3}}
+     * \endcode
      */
-    template <class E, class S>
-    inline auto dynamic_view(E&& e, S&& slices)
+    template <class E>
+    inline auto dynamic_view(E&& e, const slice_vector& slices)
     {
         // Compute dimension
         std::size_t dimension = e.dimension();
 
         for (const auto& el : slices)
         {
-            if (el[0] >= 0 && el[1] == 0)
+            if (xtl::get_if<xt::xnewaxis_tag>(&el) != nullptr)
             {
-                // treat this like a single integral and remove from shape
-                --dimension;
+                dimension++;
             }
-            else if (el[0] == -1 && el[1] == 0)
+            else if (xtl::get_if<int>(&el) != nullptr)
             {
-                // treat this like a new axis
-                ++dimension;
+                dimension--;
             }
         }
 
@@ -1075,39 +862,46 @@ namespace xt
         auto old_shape = e.shape();
         auto&& old_strides = detail::get_strides(e);
 
-        std::size_t i = 0;
-        std::size_t idx = 0;
-        std::size_t newaxis_skip = 0;
+        std::size_t i = 0, idx = 0, newaxis_skip = 0;
+
+        auto slice_getter = detail::slice_getter_impl<E>(e);
 
         for (; i < slices.size(); ++i)
         {
-            if (slices[i][0] >= 0)
+            auto ptr = xtl::get_if<int>(&slices[i]);
+            if (ptr != nullptr)
             {
-                std::size_t slice0 = static_cast<std::size_t>(slices[i][0]);
-                offset += slice0 * old_strides[i];
+                std::size_t slice0 = static_cast<std::size_t>(*ptr);
+                offset += slice0 * old_strides[i - newaxis_skip];
             }
-
-            if (slices[i][1] != 0 && slices[i][2] != 0)
-            {
-                std::size_t slice1 = static_cast<std::size_t>(slices[i][1]);
-                std::size_t slice2 = static_cast<std::size_t>(slices[i][2]);
-                new_shape[idx] = slice1;
-                new_strides[idx] = slice2 * old_strides[i - newaxis_skip];
-                ++idx;
-            }
-            else if (slices[i][0] == -1)  // newaxis
+            else if (xtl::get_if<xt::xnewaxis_tag>(&slices[i]) != nullptr)
             {
                 new_shape[idx] = 1;
                 new_strides[idx] = 0;
                 ++newaxis_skip;
                 ++idx;
             }
+            else if (xtl::get_if<xt::xall_tag>(&slices[i]) != nullptr)
+            {
+                new_shape[idx] = old_shape[i - newaxis_skip];
+                new_strides[idx] = old_strides[i - newaxis_skip];
+                ++idx;
+            }
+            else
+            {
+                slice_getter.idx = i - newaxis_skip;
+                std::array<int, 3> info = xtl::visit(slice_getter, slices[i]);
+                offset += std::size_t(info[0]) * old_strides[i - newaxis_skip];
+                new_shape[idx] = std::size_t(info[1]);
+                new_strides[idx] = std::size_t(info[2]) * old_strides[i - newaxis_skip];
+                ++idx;
+            }
         }
 
-        for (; i < old_shape.size(); ++i)
+        for (; (i - newaxis_skip) < old_shape.size(); ++i)
         {
-            new_shape[idx] = old_shape[i];
-            new_strides[idx] = old_strides[i];
+            new_shape[idx] = old_shape[i - newaxis_skip];
+            new_strides[idx] = old_strides[i - newaxis_skip];
             ++idx;
         }
 
@@ -1117,6 +911,236 @@ namespace xt
         // TODO change layout type?
         return view_type(std::forward<E>(e), std::forward<decltype(data)>(data), std::move(new_shape), std::move(new_strides), offset, layout_type::dynamic);
     }
+
+    /****************************
+     * transpose implementation *
+     ****************************/
+
+    namespace detail
+    {
+        inline layout_type transpose_layout_noexcept(layout_type l) noexcept
+        {
+            layout_type result = l;
+            if (l == layout_type::row_major)
+            {
+                result = layout_type::column_major;
+            }
+            else if (l == layout_type::column_major)
+            {
+                result = layout_type::row_major;
+            }
+            return result;
+        }
+
+        inline layout_type transpose_layout(layout_type l)
+        {
+            if (l != layout_type::row_major && l != layout_type::column_major)
+            {
+                throw transpose_error("cannot compute transposed layout of dynamic layout");
+            }
+            return transpose_layout_noexcept(l);
+        }
+
+        template <class E, class S>
+        inline auto transpose_impl(E&& e, S&& permutation, check_policy::none)
+        {
+            if (sequence_size(permutation) != e.dimension())
+            {
+                throw transpose_error("Permutation does not have the same size as shape");
+            }
+
+            // permute stride and shape
+            using strides_type = typename std::decay_t<E>::strides_type;
+            strides_type temp_strides;
+            resize_container(temp_strides, e.strides().size());
+
+            using shape_type = typename std::decay_t<E>::shape_type;
+            shape_type temp_shape;
+            resize_container(temp_shape, e.shape().size());
+
+            using size_type = typename std::decay_t<E>::size_type;
+            for (std::size_t i = 0; i < e.shape().size(); ++i)
+            {
+                if (std::size_t(permutation[i]) >= e.dimension())
+                {
+                    throw transpose_error("Permutation contains wrong axis");
+                }
+                size_type perm = static_cast<size_type>(permutation[i]);
+                temp_shape[i] = e.shape()[perm];
+                temp_strides[i] = e.strides()[perm];
+            }
+
+            layout_type new_layout = layout_type::dynamic;
+            if (std::is_sorted(std::begin(permutation), std::end(permutation)))
+            {
+                // keep old layout
+                new_layout = e.layout();
+            }
+            else if (std::is_sorted(std::begin(permutation), std::end(permutation), std::greater<>()))
+            {
+                new_layout = transpose_layout_noexcept(e.layout());
+            }
+
+            using view_type = xstrided_view<xclosure_t<E>, shape_type, decltype(e.data())>;
+            return view_type(std::forward<E>(e), std::move(temp_shape), std::move(temp_strides), 0, new_layout);
+        }
+
+        template <class E, class S>
+        inline auto transpose_impl(E&& e, S&& permutation, check_policy::full)
+        {
+            // check if axis appears twice in permutation
+            for (std::size_t i = 0; i < sequence_size(permutation); ++i)
+            {
+                for (std::size_t j = i + 1; j < sequence_size(permutation); ++j)
+                {
+                    if (permutation[i] == permutation[j])
+                    {
+                        throw transpose_error("Permutation contains axis more than once");
+                    }
+                }
+            }
+            return transpose_impl(std::forward<E>(e), std::forward<S>(permutation), check_policy::none());
+        }
+
+        template <class E, class S, std::enable_if_t<has_raw_data_interface<std::decay_t<E>>::value>* = nullptr>
+        inline void compute_transposed_strides(E&& e, const S&, S& strides)
+        {
+            std::copy(e.strides().rbegin(), e.strides().rend(), strides.begin());
+        }
+
+        template <class E, class S, std::enable_if_t<!has_raw_data_interface<std::decay_t<E>>::value>* = nullptr>
+        inline void compute_transposed_strides(E&&, const S& shape, S& strides)
+        {
+            layout_type l = transpose_layout(std::decay_t<E>::static_layout);
+            compute_strides(shape, l, strides);
+        }
+    }
+
+    /**
+     * Returns a transpose view by reversing the dimensions of xexpression e
+     * @param e the input expression
+     */
+    template <class E>
+    inline auto transpose(E&& e) noexcept
+    {
+        using shape_type = typename std::decay_t<E>::shape_type;
+        shape_type shape;
+        resize_container(shape, e.shape().size());
+        std::copy(e.shape().rbegin(), e.shape().rend(), shape.begin());
+
+        shape_type strides;
+        resize_container(strides, e.shape().size());
+        detail::compute_transposed_strides(e, shape, strides);
+
+        layout_type new_layout = detail::transpose_layout_noexcept(e.layout());
+
+        decltype(auto) data = detail::get_data(e);
+        using view_type = xstrided_view<xclosure_t<E>, shape_type, decltype(data)>;
+        return view_type(std::forward<E>(e), std::forward<decltype(data)>(data), std::move(shape), std::move(strides), 0, new_layout);
+    }
+
+    /**
+     * Returns a transpose view by permuting the xexpression e with @p permutation.
+     * @param e the input expression
+     * @param permutation the sequence containing permutation
+     * @param check_policy the check level (check_policy::full() or check_policy::none())
+     * @tparam Tag selects the level of error checking on permutation vector defaults to check_policy::none.
+     */
+    template <class E, class S, class Tag = check_policy::none>
+    inline auto transpose(E&& e, S&& permutation, Tag check_policy = Tag())
+    {
+        return detail::transpose_impl(std::forward<E>(e), std::forward<S>(permutation), check_policy);
+    }
+
+    /// @cond DOXYGEN_INCLUDE_SFINAE
+#ifdef X_OLD_CLANG
+    template <class E, class I, class Tag = check_policy::none>
+    inline auto transpose(E&& e, std::initializer_list<I> permutation, Tag check_policy = Tag())
+    {
+        dynamic_shape<I> perm(permutation);
+        return detail::transpose_impl(std::forward<E>(e), std::move(perm), check_policy);
+    }
+#else
+    template <class E, class I, std::size_t N, class Tag = check_policy::none>
+    inline auto transpose(E&& e, const I(&permutation)[N], Tag check_policy = Tag())
+    {
+        return detail::transpose_impl(std::forward<E>(e), permutation, check_policy);
+    }
+#endif
+    /// @endcond
+
+    /***************************
+     * ravel and flatten views *
+     ***************************/
+
+    namespace detail
+    {
+        template <class E, class D>
+        inline auto build_ravel_view(E&& e, D&& data)
+        {
+            using shape_type = static_shape<std::size_t, 1>;
+            using view_type = xstrided_view<xclosure_t<E>, shape_type, D>;
+
+            shape_type new_shape, new_strides;
+            new_shape[0] = e.size();
+            new_strides[0] = std::size_t(1);
+            std::size_t offset = detail::get_offset(e);
+
+            return view_type(std::forward<E>(e), std::forward<D>(data),
+                std::move(new_shape), std::move(new_strides),
+                offset, layout_type::dynamic);
+        }
+
+        template <bool same_layout>
+        struct ravel_impl
+        {
+            template <class E>
+            inline static auto run(E&& e)
+            {
+                return build_ravel_view(std::forward<E>(e), detail::get_data(e));
+            }
+        };
+
+        template <>
+        struct ravel_impl<false>
+        {
+            template <class E>
+            inline static auto run(E&& e)
+            {
+                using proxy_type = expression_adaptor<xclosure_t<E>>;
+                using shape_type = typename std::decay_t<E>::shape_type;
+                shape_type strides;
+                resize_container(strides, e.shape().size());
+                layout_type l = detail::transpose_layout(e.layout());
+                compute_strides(e.shape(), l, strides);
+                return build_ravel_view(e, proxy_type(e, std::move(strides), l));
+            }
+        };
+    }
+
+    /**
+     * Returns a flatten view of the given expression. No copy is made.
+     * @param e the input expression
+     * @tparam L the layout used to read the elements of e
+     * @tparam E the type of the expression
+     */
+    template <layout_type L, class E>
+    inline auto ravel(E&& e)
+    {
+        return detail::ravel_impl<std::decay_t<E>::static_layout == L>::run(std::forward<E>(e));
+    }
+
+    /**
+     * Returns a flatten view of the given expression. No copy is made.
+     * The layout used to read the elements is the one of e.
+     * @param e the input expression
+     * @tparam E the type of the expression
+     */
+    template <class E>
+    inline auto flatten(E&& e)
+    {
+        return ravel<std::decay_t<E>::static_layout>(std::forward<E>(e));
+    }
 }
 
 #endif
