@@ -11,7 +11,7 @@
 
 #define XTENSOR_VERSION_MAJOR 0
 #define XTENSOR_VERSION_MINOR 15
-#define XTENSOR_VERSION_PATCH 4
+#define XTENSOR_VERSION_PATCH 5
 
 // DETECT 3.6 <= clang < 3.8 for compiler bug workaround.
 #ifdef __clang__
@@ -33,7 +33,7 @@
 
 #ifndef DEFAULT_ALLOCATOR
 #ifdef XTENSOR_USE_XSIMD
-#include "xsimd/xsimd.hpp"
+#include <xsimd/xsimd.hpp>
 #define DEFAULT_ALLOCATOR(T) \
     xsimd::aligned_allocator<T, XSIMD_DEFAULT_ALIGNMENT>
 #else
