@@ -16,8 +16,8 @@
 #include <type_traits>
 #include <utility>
 
-#include "xtl/xclosure.hpp"
-#include "xtl/xsequence.hpp"
+#include <xtl/xclosure.hpp>
+#include <xtl/xsequence.hpp>
 
 #include "xbroadcast.hpp"
 #include "xcontainer.hpp"
@@ -146,7 +146,7 @@ namespace xt
         const_reference element(It first, It last) const;
 
         template <class ST>
-        bool broadcast_shape(ST& shape) const;
+        bool broadcast_shape(ST& shape, bool reuse_cache = false) const;
 
         template <class ST>
         bool is_trivial_broadcast(const ST& strides) const;
@@ -161,12 +161,16 @@ namespace xt
         template <class ST>
         const_stepper stepper_end(const ST& shape, layout_type l) const;
 
+        template <class T = xexpression_type>
+        std::enable_if_t<has_raw_data_interface<T>::value, typename T::container_type&>
+        data();
+
         template <class T = xexpression_type>
         std::enable_if_t<has_raw_data_interface<T>::value, const typename T::container_type&>
         data() const;
 
         template <class T = xexpression_type>
-        std::enable_if_t<has_raw_data_interface<T>::value, const typename T::strides_type>
+        std::enable_if_t<has_raw_data_interface<T>::value, const strides_type&>
         strides() const;
 
         template <class T = xexpression_type>
@@ -199,6 +203,10 @@ namespace xt
         CT m_e;
         slice_type m_slices;
         inner_shape_type m_shape;
+        mutable strides_type m_strides;
+        mutable bool m_strides_computed;
+
+        strides_type compute_strides() const;
 
         template <typename std::decay_t<CT>::size_type... I, class... Args>
         reference access_impl(std::index_sequence<I...>, Args... args);
@@ -352,7 +360,8 @@ namespace xt
     template <class CTA, class FSL, class... SL>
     inline xview<CT, S...>::xview(CTA&& e, FSL&& first_slice, SL&&... slices) noexcept
         : m_e(std::forward<CTA>(e)), m_slices(std::forward<FSL>(first_slice), std::forward<SL>(slices)...),
-          m_shape(xtl::make_sequence<shape_type>(m_e.dimension() - integral_count<S...>() + newaxis_count<S...>(), 0))
+          m_shape(xtl::make_sequence<shape_type>(m_e.dimension() - integral_count<S...>() + newaxis_count<S...>(), 0)),
+          m_strides_computed(false)
     {
         auto func = [](const auto& s) noexcept { return get_size(s); };
         for (size_type i = 0; i != dimension(); ++i)
@@ -595,6 +604,14 @@ namespace xt
      * container). ``xt::eval`` will make sure that the underlying xexpression is
      * on a realized container.
      */
+    template <class CT, class... S>
+    template <class T>
+    inline auto xview<CT, S...>::data() ->
+        std::enable_if_t<has_raw_data_interface<T>::value, typename T::container_type&>
+    {
+        return m_e.data();
+    }
+
     template <class CT, class... S>
     template <class T>
     inline auto xview<CT, S...>::data() const ->
@@ -609,29 +626,14 @@ namespace xt
     template <class CT, class... S>
     template <class T>
     inline auto xview<CT, S...>::strides() const ->
-        std::enable_if_t<has_raw_data_interface<T>::value, const typename T::strides_type>
+        std::enable_if_t<has_raw_data_interface<T>::value, const strides_type&>
     {
-        using strides_type = typename T::strides_type;
-        strides_type strides = xtl::make_sequence<strides_type>(m_e.dimension() - integral_count<S...>(), 0);
-
-        auto func = [](const auto& s) { return xt::step_size(s); };
-        size_type i = 0, idx;
-
-        for (; i < sizeof...(S); ++i)
+        if (!m_strides_computed)
         {
-            idx = integral_skip<S...>(i);
-            if (idx >= sizeof...(S))
-            {
-                break;
-            }
-            strides[i] = m_e.strides()[idx] * apply<size_type>(idx, func, m_slices);
+            m_strides = compute_strides();
+            m_strides_computed = true;
         }
-        for (; i < strides.size(); ++i)
-        {
-            strides[i] = m_e.strides()[idx++];
-        }
-
-        return strides;
+        return m_strides;
     }
 
     /**
@@ -708,7 +710,7 @@ namespace xt
      */
     template <class CT, class... S>
     template <class ST>
-    inline bool xview<CT, S...>::broadcast_shape(ST& shape) const
+    inline bool xview<CT, S...>::broadcast_shape(ST& shape, bool) const
     {
         return xt::broadcast_shape(m_shape, shape);
     }
@@ -726,6 +728,28 @@ namespace xt
     }
     //@}
 
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::compute_strides() const -> strides_type
+    {
+        strides_type strides = xtl::make_sequence<strides_type>(dimension(), 0);
+
+        auto func = [](const auto& s) { return xt::step_size(s); };
+        size_type i = 0;
+
+        for (; i < sizeof...(S); i++)
+        {
+            strides[i - integral_count_before<S...>(i)] =
+                m_e.strides()[i - newaxis_count_before<S...>(i)] *
+                apply<size_type>(i, func, m_slices);
+        }
+        for (; i < strides.size(); ++i)
+        {
+            strides[i - integral_count<S...>()] =
+                m_e.strides()[i - newaxis_count<S...>()];
+        }
+        return strides;
+    }
+
     template <class CT, class... S>
     template <typename std::decay_t<CT>::size_type... I, class... Args>
     inline auto xview<CT, S...>::access_impl(std::index_sequence<I...>, Args... args) -> reference
